package org.netkernel.json.core.endpoint;

import java.io.InputStream;
import java.util.Iterator;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.netkernel.json.core.util.HDS2Utils;
import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class JSONToHDS extends StandardAccessorImpl
{
	public static final String ARRAY_INDICATOR_SUFFIX=JSONFromHDS.ARRAY_INDICATOR_SUFFIX;
	
	public JSONToHDS()
	{	declareThreadSafe();
		declareArgument(new SourcedArgumentMetaImpl("operand","input2 JSON Object",null,new Class[]{JSONObject.class,JSONArray.class}));
		declareSourceRepresentation(IHDSNode.class);
		declareSourceRepresentation(IHDSDocument.class);
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		Object rawOperand=aContext.source("arg:operand");
		Class required=aContext.getThisRequest().getRepresentationClass();
		if (rawOperand instanceof IReadableBinaryStreamRepresentation && required==IHDSDocument.class)
		{	InputStream is=((IReadableBinaryStreamRepresentation)rawOperand).getInputStream();
			IHDSDocument doc=HDS2Utils.parseHDS(is);
			aContext.createResponseFrom(doc);
		}
		else
		{	if (! (rawOperand instanceof JSONObject))
			{	rawOperand=aContext.transrept(rawOperand, JSONObject.class);
			}
			JSONObject json=(JSONObject)rawOperand;
			
			HDSBuilder b=new HDSBuilder();
			recurseJSONToHDS(b,json);
			aContext.createResponseFrom(b.getRoot());
		}
	}
	
	private static void recurseJSONToHDS(HDSBuilder b, JSONObject j) throws JSONException
	{	for (Iterator<String> i=j.keys(); i.hasNext(); )
		{	String key=i.next();
			Object object=j.get(key);
			writeJSONToHDS(b,key,object);
		}
	}
	
	private static void recurseJSONArrayToHDS(HDSBuilder b, String name, JSONArray j) throws JSONException
	{	int l=j.length();
		for (int i=0; i<l; i++)
		{	Object object=j.get(i);
			writeJSONToHDS(b,name,object);
		}
	}
	
	private static void writeJSONToHDS(HDSBuilder b,String key, Object object) throws JSONException
	{	if (object instanceof JSONObject)
		{	b.pushNode(key);
			recurseJSONToHDS(b,(JSONObject)object);
			b.popNode();
		}
		else if (object instanceof JSONArray)
		{	b.pushNode(key+ARRAY_INDICATOR_SUFFIX);
			recurseJSONArrayToHDS(b,key,(JSONArray)object);
			b.popNode();
		}
		else if (object instanceof Number || object instanceof String || object instanceof Boolean)
		{	b.addNode(key,object);
		}
		else if (object==JSONObject.NULL)
		{	b.addNode(key,null);
		}
	}	
}
